package com.joelhelkala.watcherGui.frames;
import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.util.*;
import java.util.Timer;

import javax.swing.*;
import javax.swing.border.Border;

import com.joelhelkala.watcherGui.User.User;
import com.joelhelkala.watcherGui.httpRequests.HttpRequests;

/*
 * Login page which is displayed when the application is ran
 */
public class LoginPage implements ActionListener{
	private JFrame frame;
	private JButton loginButton;
	private JButton newButton;
	private JTextField userIDField;
	private JLabel titleLabel;
	private JLabel messageLabel;
	private JLabel retryLabel;
	private JPasswordField passwordField;
	private JPanel bottomPanel;
	private JLabel imageLabel;
	private ImageIcon lightImageOff = new ImageIcon("src/main/resources/images/lightbulb_off.png");
	private ImageIcon lightImageOn = new ImageIcon("src/main/resources/images/lightbulb_on.png");
	
	private final int width = 800, height = 420;
	private final int[] retry_intervals = {5, 15, 30, 60, 180};
	private int retry_count = 0;
	
	// Constructor
	public LoginPage(){
		frame = new JFrame();
		frame.getContentPane().setLayout(null);
		loginButton = new JButton("Sign In");
		newButton = new JButton("Sign Up");
		newButton.setBackground(Color.GREEN);
		userIDField = new JTextField();
		titleLabel = new JLabel("Sign In");
		messageLabel = new JLabel();
		retryLabel = new JLabel();
		bottomPanel = new JPanel();
		imageLabel = new JLabel();
		
		JPanel topPanel = new JPanel();
		JLabel emailLabel = new JLabel("Email");
		JLabel userPasswordLabel = new JLabel("Password");		
		
		emailLabel.setBounds(50,100,75,25);
		emailLabel.setForeground(Color.white);
		userPasswordLabel.setBounds(50,137,75,25);
		userPasswordLabel.setForeground(Color.white);
		titleLabel.setBounds(175,0,250,35);
		titleLabel.setFont(new Font(null, Font.BOLD, 25));
		titleLabel.setForeground(Color.white);
		messageLabel.setBounds(125,240,250,35);
		messageLabel.setFont(new Font(null,Font.ITALIC,15));
		retryLabel.setBounds(125, 255, 250, 35);
		retryLabel.setFont(new Font(null, Font.ITALIC, 15));
		retryLabel.setForeground(Color.red);
		imageLabel.setBackground(Color.black);
		imageLabel.setBounds(80,0, width, height-100);
		
		userIDField.setBounds(125,100,200,25);
		loginButton.setBounds(225,188,100,25);
		loginButton.setFocusable(false);
		loginButton.addActionListener(this);
		
		passwordField = new JPasswordField();
		passwordField.setBounds(125, 136, 200, 26);
		
		newButton.setBounds(width/4-117/2, height-29*3, 117, 29);
		newButton.setBorder(new RoundedBorder(10));
		newButton.setForeground(Color.orange);
		
		newButton.addActionListener(this);

		bottomPanel.setBackground(Color.black);
		bottomPanel.setBounds(0,0,width, height);
		bottomPanel.setLayout(null);
		
		setLightBulb();
		
		topPanel.setLayout(null);
		topPanel.add(emailLabel);
		topPanel.add(userPasswordLabel);
		topPanel.add(titleLabel);
		topPanel.add(messageLabel);
		topPanel.add(retryLabel);
		topPanel.add(userIDField);
		topPanel.add(loginButton);
		topPanel.add(passwordField);
		bottomPanel.add(newButton);
		bottomPanel.add(imageLabel);
		
		topPanel.setBackground(Color.black);
		topPanel.setBounds(width / 2, 60, width/2-30, height-140);
		
		bottomPanel.add(topPanel);
		frame.getContentPane().add(bottomPanel);
		
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(width,height);
		frame.setMinimumSize(new Dimension(800, 420));
		frame.setMaximumSize(new Dimension(800, 420));
		frame.getContentPane().setLayout(null);
		
		frame.setLocationRelativeTo(null);
		frame.setVisible(true);
		frame.getRootPane().setDefaultButton(loginButton);
		
		CheckBackendStatus();
	}
	
	// Sets a lightbulb image on login window
	private void setLightBulb() {
		imageLabel.setIcon(resizeIcon(lightImageOff, 255, 343));
	}
	
	// Sets the lightbulb image on
	private void setLightBulbOn() {
		imageLabel.setIcon(resizeIcon(lightImageOn, 255, 343));
	}
	
	// Resize the given image to given width and height
	protected static Icon resizeIcon(ImageIcon icon, int resizedWidth, int resizedHeight) {
	    Image img = icon.getImage();  
	    Image resizedImage = img.getScaledInstance(resizedWidth, resizedHeight,  java.awt.Image.SCALE_SMOOTH);  
	    return new ImageIcon(resizedImage);
	}
	
	// Sends a ping to backend to check connection
	private void CheckBackendStatus() {
		try {
			if(!HttpRequests.PingRequest()) {
				SetError("Can't connect to server");
				LoopConnect();
				loginButton.setEnabled(false);
			} else {
				messageLabel.setText(null);
				retryLabel.setText(null);
				loginButton.setEnabled(true);
				setLightBulbOn();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/*
	 * Tries to connect to backend in a loop
	 * with an increasing timer
	 */
	private void LoopConnect() {
		final Timer timer = new Timer();

        timer.scheduleAtFixedRate(new TimerTask() {
        	int i = retry_intervals[retry_count];

            public void run() {
                retryLabel.setText("Retry in: " + i);
                i--;

                if (i < 0) {
                    timer.cancel();
                    if(retry_count < retry_intervals.length-1) retry_count++;
                    CheckBackendStatus();
                }
            }
        }, 0, 1000);
	}
	
	// Set error message to the screen
	private void SetError(String error) {
		messageLabel.setForeground(Color.red);
		messageLabel.setText(error);	
	}
	
	// Clears the email and password fields
	private void ClearFields() {
		userIDField.setText("");
		passwordField.setText("");
		userIDField.requestFocus();
	}
	
	/*
	 * When button is pressed with actionlistener attached
	 */
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==loginButton) {
			String email = userIDField.getText();
			String password = String.valueOf(passwordField.getPassword());
			if(email.equals("") || password.equals("")) return;
			
			User.setEmail(email);
			boolean valid = HttpRequests.LoginRequest(email, password);
			
			if (valid) {
				frame.dispose();
				WelcomePage welcomePage = new WelcomePage();
			} else {						
				SetError("Invalid username or password");
				ClearFields();
			}
		}
		else if(e.getSource() == newButton) {
			frame.dispose();
			RegisterPage register = new RegisterPage();
		}
	}
	
	// Make a button have rounded edges 
	// https://stackoverflow.com/questions/423950/rounded-swing-jbutton-using-java
	private static class RoundedBorder implements Border {
	    private int radius;

	    RoundedBorder(int radius) {
	        this.radius = radius;
	    }


	    public Insets getBorderInsets(Component c) {
	        return new Insets(this.radius+1, this.radius+1, this.radius+2, this.radius);
	    }


	    public boolean isBorderOpaque() {
	        return true;
	    }


	    public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
	        g.drawRoundRect(x, y, width-1, height-1, radius, radius);
	    }
	}
}

