package com.joelhelkala.watcherGui.httpRequests;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.charset.StandardCharsets;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.joelhelkala.watcherGui.User.User;

/*
 * Class to handle httpRequests to the server
 */
public class HttpRequests {
	
	private final static String address = "http://localhost:8080/api/v1";
	private static HttpURLConnection connection;
	
	// Sends a simple ping GET to the server to check if it's up
	public static boolean PingRequest() throws IOException{
		URL url = new URL(address + "/ping");
        connection = (HttpURLConnection) url.openConnection();
        connection.setRequestProperty("accept", "application/json");
        connection.setRequestMethod("GET");
        
        try {
        	if(connection.getResponseCode() == HttpURLConnection.HTTP_OK) {
        		connection.disconnect();
        		return true;
        	}
		} catch (IOException e) {}
        finally {
        	connection.disconnect();
        }
		return false;
	}
	
	/*
	 * Sends a POST request to register a new user to database
	 * Server sends a token if the information given are valid
	 * This token must be confirmed under certain time
	 */
	public static JSONObject RegisterUser(String firstname, String lastname, String email, String password) {
		JSONObject response_json = null;
		
		try {
			URL url = new URL(address+"/registration");
			HttpURLConnection con = (HttpURLConnection)url.openConnection();
			con.setRequestMethod("POST");
			con.setRequestProperty("Content-Type", "application/json; utf-8");
			con.setRequestProperty("Accept", "application/json");
			con.setDoOutput(true);
			
			JSONObject obj = new JSONObject();
			obj.put("firstName", firstname);
			obj.put("lastName", lastname);
			obj.put("email", email);
			obj.put("password", password);
			
			String jsonString = obj.toString();
			
			try(OutputStream os = con.getOutputStream()) {
			    byte[] input = jsonString.getBytes("utf-8");
			    os.write(input, 0, input.length);			
			}
			
			int status = con.getResponseCode();
			if (status > 299) {
				try(BufferedReader br = new BufferedReader(
				  new InputStreamReader(con.getErrorStream(), "utf-8"))) {
				    StringBuilder response = new StringBuilder();
				    String responseLine = null;
				    while ((responseLine = br.readLine()) != null) {
				        response.append(responseLine.trim());
				    }
				    response_json = new JSONObject(response.toString());
				    response_json.put("status", status);
				}
			} else {
				try(BufferedReader br = new BufferedReader(
				  new InputStreamReader(con.getInputStream(), "utf-8"))) {
				    StringBuilder response = new StringBuilder();
				    String responseLine = null;
				    while ((responseLine = br.readLine()) != null) {
				        response.append(responseLine.trim());
				    }
				    response_json = new JSONObject(response.toString());
				    response_json.put("status", status);
				}
			}
			con.disconnect();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return response_json;
	}
	
	/*
	 * Send a login POST request to server
	 * TODO: Get a session token from the server which will be used for communicating with endpoints
	 */
	public static boolean LoginRequest(String user, String password) {
		try {
			String urlParameters  = "username=" + user + "&password=" + password;
			byte[] postData = urlParameters.getBytes( StandardCharsets.UTF_8 );
			StringBuilder response = new StringBuilder();
			URL url = new URL(address+"/login");
			HttpURLConnection con = (HttpURLConnection)url.openConnection();
			con.setRequestMethod("POST");
			con.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
			con.setRequestProperty("charset", "utf-8");
			con.setRequestProperty("Accept", "application/json");
			con.setRequestProperty("Content-Length", Integer.toString(postData.length));
			con.setUseCaches(false);
			con.setDoOutput(true);
			
			try(DataOutputStream wr = new DataOutputStream(con.getOutputStream())) {
			   wr.write( postData );
			}
			
			try(BufferedReader br = new BufferedReader(
			  new InputStreamReader(con.getInputStream(), "utf-8"))) {
			    String responseLine = null;
			    while ((responseLine = br.readLine()) != null) {
			        response.append(responseLine.trim());
			    }
			}
			JSONObject object = new JSONObject(response.toString());
			User.setToken(object.get("access_token").toString());
			con.disconnect();
			return true;
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (JSONException e) {
			e.printStackTrace();
		}
		return false;
	}
	
	public static JSONArray parse(String responseBody) {
		JSONArray body = new JSONArray(responseBody);
		for(int i = 0; i < body.length(); i++) {
			JSONObject obj = body.getJSONObject(i);
			int code = obj.getInt("code");
			int id = obj.getInt("id");
			String user = obj.getString("user");
			System.out.println(code + " " + id + " " + user);
		}
		return body;
	}
}
